@@ -1,9 +1,9 @@
-/* Copyright (C) 1988, 1989 Free Software Foundation, Inc.
+/* Copyright (C) 1988, 1989, 1990, 1991, 1992, 1993 Free Software Foundation, Inc.
 This file is part of GNU Make.
 
 GNU Make is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 1, or (at your option)
+the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 GNU Make is distributed in the hope that it will be useful,
@@ -84,7 +84,7 @@ collapse_continuations (line)
       if (backslash)
 	{
 	  in = next_token (in);
-	  while (out > line && (out[-1] == ' ' || out[-1] == '\t'))
+	  while (out > line && isblank (out[-1]))
 	    --out;
 	  *out++ = ' ';
 	}
@@ -116,21 +116,17 @@ collapse_continuations (line)
 }
 
 
-/* Remove comments, backslash-newline combinations and whitespace
-   following such combinations from LINE.
-   This is done by copying the text at LINE into itself.  */
+/* Remove comments from LINE.
+   This is done by copying the text at LINE onto itself.  */
 
 void
-collapse_line (line)
+remove_comments (line)
      char *line;
 {
   register char *p, *p2;
   register int backslash;
   register unsigned int bs_write;
 
-  /* Collapse backslash-newline continuations.  */
-  collapse_continuations (line);
-
   while (1)
     {
       p = index (line, '#');
@@ -197,6 +193,21 @@ concat (s1, s2, s3)
   return result;
 }
 
+/* Print a message on stdout.  */
+
+void
+message (s1, s2, s3, s4, s5, s6)
+     char *s1, *s2, *s3, *s4, *s5, *s6;
+{
+  if (makelevel == 0)
+    printf ("%s: ", program);
+  else
+    printf ("%s[%u]: ", program, makelevel);
+  printf (s1, s2, s3, s4, s5, s6);
+  putchar ('\n');
+  fflush (stdout);
+}
+
 /* Print an error message and exit.  */
 
 /* VARARGS1 */
@@ -205,9 +216,9 @@ fatal (s1, s2, s3, s4, s5, s6)
      char *s1, *s2, *s3, *s4, *s5, *s6;
 {
   if (makelevel == 0)
-    fprintf (stderr, "%s: ", program);
+    fprintf (stderr, "%s: *** ", program);
   else
-    fprintf (stderr, "%s[%u]: ", program, makelevel);
+    fprintf (stderr, "%s[%u]: *** ", program, makelevel);
   fprintf (stderr, s1, s2, s3, s4, s5, s6);
   fputs (".  Stop.\n", stderr);
 
@@ -226,11 +237,36 @@ error (s1, s2, s3, s4, s5, s6)
     fprintf (stderr, "%s: ", program);
   else
     fprintf (stderr, "%s[%u]: ", program, makelevel);
-  fprintf (stderr, s1, s2, s3, s4, s5);
+  fprintf (stderr, s1, s2, s3, s4, s5, s6);
   putc ('\n', stderr);
   fflush (stderr);
 }
 
+void
+makefile_error (file, lineno, s1, s2, s3, s4, s5, s6)
+     char *file;
+     unsigned int lineno;
+     char *s1, *s2, *s3, *s4, *s5, *s6;
+{
+  fprintf (stderr, "%s:%u: ", file, lineno);
+  fprintf (stderr, s1, s2, s3, s4, s5, s6);
+  putc ('\n', stderr);
+  fflush (stderr);
+}
+
+void
+makefile_fatal (file, lineno, s1, s2, s3, s4, s5, s6)
+     char *file;
+     unsigned int lineno;
+     char *s1, *s2, *s3, *s4, *s5, *s6;
+{
+  fprintf (stderr, "%s:%u: *** ", file, lineno);
+  fprintf (stderr, s1, s2, s3, s4, s5, s6);
+  fputs (".  Stop.\n", stderr);
+
+  die (1);
+}
+
 /* Print an error message from errno.  */
 
 void
@@ -264,13 +300,15 @@ pfatal_with_name (name)
 }
 
 /* Like malloc but get fatal error if memory is exhausted.  */
-extern char *malloc (), *realloc ();
+
+#undef xmalloc
+#undef xrealloc
 
 char *
 xmalloc (size)
      unsigned int size;
 {
-  char *result = malloc (size);
+  char *result = (char *) malloc (size);
   if (result == 0)
     fatal ("virtual memory exhausted");
   return result;
@@ -282,7 +320,7 @@ xrealloc (ptr, size)
      char *ptr;
      unsigned int size;
 {
-  char *result = realloc (ptr, size);
+  char *result = (char *) realloc (ptr, size);
   if (result == 0)
     fatal ("virtual memory exhausted");
   return result;
@@ -352,7 +390,7 @@ end_of_token (s)
   register char *p = s;
   register int backslash = 0;
 
-  while (*p != '\0' && (backslash || (*p != ' ' && *p != '\t' && *p != '\f')))
+  while (*p != '\0' && (backslash || !isblank (*p)))
     {
       if (*p++ == '\\')
 	{
@@ -378,7 +416,7 @@ next_token (s)
 {
   register char *p = s;
 
-  while (*p == ' ' || *p == '\t' || *p == '\f')
+  while (isblank (*p))
     ++p;
   return p;
 }
@@ -442,3 +480,181 @@ dep_name (dep)
   return dep->name == 0 ? dep->file->name : dep->name;
 }
 #endif
+
+#ifdef	GETLOADAVG_PRIVILEGED
+
+#ifndef	HAVE_UNISTD_H
+extern int getuid (), getgid (), geteuid (), getegid ();
+extern int setuid (), setgid ();
+#ifdef	HAVE_SETREUID
+extern int setreuid ();
+#endif	/* Have setreuid.  */
+#ifdef	HAVE_SETREGID
+extern int setregid ();
+#endif	/* Have setregid.  */
+#endif	/* No <unistd.h>.  */
+
+/* Keep track of the user and group IDs for user- and make- access.  */
+static int user_uid = -1, user_gid = -1, make_uid = -1, make_gid = -1;
+#define	access_inited	(user_uid != -1)
+static enum { make, user } current_access;
+
+static void
+init_access ()
+{
+  user_uid = getuid ();
+  user_gid = getgid ();
+
+  make_uid = geteuid ();
+  make_gid = getegid ();
+
+  /* Do these ever fail?  */
+  if (user_uid == -1 || user_gid == -1 || make_uid == -1 || make_gid == -1)
+    pfatal_with_name ("get{e}[gu]id");
+
+  current_access = make;
+}
+
+#endif	/* GETLOADAVG_PRIVILEGED */
+
+/* Give the process appropriate permissions for access to
+   user data (i.e., to stat files, or to spawn a child process).  */
+void
+user_access ()
+{
+#ifdef	GETLOADAVG_PRIVILEGED
+
+  if (!access_inited)
+    init_access ();
+
+  if (current_access == user)
+    return;
+
+  /* We are in "make access" mode.  This means that the effective user and
+     group IDs are those of make (if it was installed setuid or setgid).
+     We now want to set the effective user and group IDs to the real IDs,
+     which are the IDs of the process that exec'd make.  */
+
+#ifndef	HAVE_SETREUID
+
+  /* System V has only the setuid/setgid calls to set user/group IDs.
+     There is an effective ID, which can be set by setuid/setgid.
+     It can be set (unless you are root) only to either what it already is
+     (returned by geteuid/getegid, now in make_uid/make_gid),
+     the real ID (return by getuid/getgid, now in user_uid/user_gid),
+     or the saved set ID (what the effective ID was before this set-ID
+     executable (make) was exec'd).  */
+
+  if (setuid (user_uid) < 0)
+    pfatal_with_name ("user_access: setuid");
+
+#else
+
+  /* In 4BSD, the setreuid/setregid calls set both the real and effective IDs.
+     They may be set to themselves or each other.  So you have two alternatives
+     at any one time.  If you use setuid/setgid, the effective will be set to
+     the real, leaving only one alternative.  Using setreuid/setregid, however,
+     you can toggle between your two alternatives by swapping the values in a
+     single setreuid or setregid call.  */
+
+  if (setreuid (make_uid, user_uid) < 0)
+    pfatal_with_name ("user_access: setreuid");
+
+#endif
+
+#ifndef	HAVE_SETREGID
+  if (setgid (user_gid) < 0)
+    pfatal_with_name ("user_access: setgid");
+#else
+  if (setregid (make_gid, user_gid) < 0)
+    pfatal_with_name ("user_access: setregid");
+#endif
+
+  current_access = user;
+
+#endif	/* GETLOADAVG_PRIVILEGED */
+}
+
+/* Give the process appropriate permissions for access to
+   make data (i.e., the load average).  */
+void
+make_access ()
+{
+#ifdef	GETLOADAVG_PRIVILEGED
+
+  if (!access_inited)
+    init_access ();
+
+  if (current_access == make)
+    return;
+
+  /* See comments in user_access, above.  */
+
+#ifndef	HAVE_SETREUID
+  if (setuid (make_uid) < 0)
+    pfatal_with_name ("make_access: setuid");
+#else
+  if (setreuid (user_uid, make_uid) < 0)
+    pfatal_with_name ("make_access: setreuid");
+#endif
+
+#ifndef	HAVE_SETREGID
+  if (setgid (make_gid) < 0)
+    pfatal_with_name ("make_access: setgid");
+#else
+  if (setregid (user_gid, make_gid) < 0)
+    pfatal_with_name ("make_access: setregid");
+#endif
+
+  current_access = make;
+
+#endif	/* GETLOADAVG_PRIVILEGED */
+}
+
+/* Give the process appropriate permissions for a child process.
+   This is like user_access, but you can't get back to make_access.  */
+void
+child_access ()
+{
+#ifdef	GETLOADAVG_PRIVILEGED
+
+  /* Set both the real and effective UID and GID to the user's.
+     They cannot be changed back to make's.  */
+
+#ifndef	HAVE_SETREUID
+  if (setuid (user_uid) < 0)
+    pfatal_with_name ("child_access: setuid");
+#else
+  if (setreuid (user_uid, user_uid) < 0)
+    pfatal_with_name ("child_access: setreuid");
+#endif
+
+#ifndef	HAVE_SETREGID
+  if (setgid (user_gid) < 0)
+    pfatal_with_name ("child_access: setgid");
+#else
+  if (setregid (user_gid, user_gid) < 0)
+    pfatal_with_name ("child_access: setregid");
+#endif
+
+#endif	/* GETLOADAVG_PRIVILEGED */
+}
+
+#ifdef NEED_GET_PATH_MAX
+unsigned int
+get_path_max ()
+{
+  static unsigned int value;
+
+  if (value == 0)
+    {
+      long int x = pathconf ("/", _PC_PATH_MAX);
+      if (x > 0)
+	value = x;
+      else
+	return MAXPATHLEN;
+    }
+
+  return value;
+}
+#endif
